// SPDX-License-Identifier: MIT

use crate::common;
use anyhow::Result;
use parking_lot::Mutex;
use serde::{Deserialize, Serialize};
use std::path::Path;
use std::sync::Arc;
use tokio::fs;
use tracing::warn;

const STATIC_CUSTOMIZE_JSON: &str = "/etc/atmark/abos_web/customize.json";

#[derive(Clone, Deserialize, Serialize, Debug)]
pub struct SideMenu {
    pub href_path: String,
    pub text: String,
    pub icon_path: String,
    pub description: String,
}

#[derive(Clone, Deserialize, Serialize, Debug)]
pub struct Customize {
    pub header_title: String,
    pub side_menu: Vec<SideMenu>,
    pub color: String,
}

impl Customize {
    pub fn get_page_title(&self, href: &str) -> Option<String> {
        self.side_menu
            .iter()
            .find(|menu| menu.href_path == href)
            .filter(|menu| !menu.text.is_empty())
            .map(|menu| menu.text.clone())
    }
}

impl Default for Customize {
    fn default() -> Self {
        Self {
            header_title: "ABOS Web".to_string(),
            side_menu: Customize::side_menu_default(),
            color: "default_color".to_string(),
        }
    }
}

#[derive(Clone, Debug)]
pub struct CustomizeConfig {
    // Arc<Mutex: Mutex cannot be used in structs that use Clone Trait,
    // so it must be wrapped in Arc.
    // Arc<Customize: This is required to shallow copy the contents of Mutex.
    pub customize: Arc<Mutex<Arc<Customize>>>,
}

impl CustomizeConfig {
    pub async fn load() -> Self {
        let customize = match fs::read_to_string(STATIC_CUSTOMIZE_JSON).await {
            Ok(contents) => serde_json::from_str(&contents).unwrap_or_else(|e| {
                warn!("parse customize.json failed: {:?}", e);
                Customize::default()
            }),
            // No such file, use default silently
            Err(e) if e.raw_os_error() == Some(2) => Customize::default(),
            Err(e) => {
                warn!("read customize.json failed: {:?}", e);
                Customize::default()
            }
        };
        CustomizeConfig {
            customize: Arc::new(Mutex::new(Arc::new(customize))),
        }
    }

    pub async fn save(&self, new_customize: Customize) -> Result<()> {
        let path = Path::new(STATIC_CUSTOMIZE_JSON);
        let serialized = serde_json::to_string(&new_customize)?;
        fs::write(path, serialized).await?;
        common::exec_command(&["persist_customize.sh"]).await?;
        *self.customize.lock() = Arc::new(new_customize);
        Ok(())
    }

    pub fn get(&self) -> Arc<Customize> {
        self.customize.lock().clone()
    }

    pub fn get_clone(&self) -> Customize {
        Customize::clone(&self.customize.lock())
    }

    pub fn reset(&self) {
        *self.customize.lock() = Arc::new(Customize::default());
    }
}

impl Customize {
    pub fn side_menu_default() -> Vec<SideMenu> {
        vec![
            SideMenu {
                href_path: "./top".to_string(),
                text: "トップページ".to_string(),
                icon_path: "".to_string(),
                description: "トップページへ戻ります。".to_string(),
            },
            SideMenu {
                href_path: "./wwan".to_string(),
                text: "WWAN設定".to_string(),
                icon_path: "../static/wwan/images/icon_wwan.png".to_string(),
                description: "WWAN通信で接続するための設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./wlan".to_string(),
                text: "WLAN設定".to_string(),
                icon_path: "../static/wlan/images/icon_wlan.png".to_string(),
                description: "WLANで接続するための設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./connection".to_string(),
                text: "各接続設定".to_string(),
                icon_path: "../static/connection/images/icon_connection.png".to_string(),
                description: "LAN等で接続するための設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./dhcp".to_string(),
                text: "DHCPサーバー設定".to_string(),
                icon_path: "../static/dhcp/images/icon_dhcp.png".to_string(),
                description: "DHCPサーバーの設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./nat".to_string(),
                text: "NAT設定".to_string(),
                icon_path: "../static/nat/images/icon_nat.png".to_string(),
                description: "NATの設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./vpn".to_string(),
                text: "VPN設定".to_string(),
                icon_path: "../static/vpn/images/icon_vpn.png".to_string(),
                description: "VPNの設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./container".to_string(),
                text: "コンテナ管理".to_string(),
                icon_path: "../static/container/images/icon_container.png".to_string(),
                description: "コンテナの管理が行なえます。".to_string(),
            },
            SideMenu {
                href_path: "./swu".to_string(),
                text: "SWUインストール".to_string(),
                icon_path: "../static/swu/images/icon_swu.png".to_string(),
                description: "SWUファイルをインストールすることができます。".to_string(),
            },
            SideMenu {
                href_path: "./time".to_string(),
                text: "時刻設定".to_string(),
                icon_path: "../static/time_settings/images/icon_time.png".to_string(),
                description: "時刻の設定が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./status".to_string(),
                text: "状態一覧".to_string(),
                icon_path: "../static/status/images/icon_status.png".to_string(),
                description: "現在の状態の確認が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./settings".to_string(),
                text: "設定管理".to_string(),
                icon_path: "../static/settings/images/icon_settings.png".to_string(),
                description: "Rest API のトークンの管理が行えます。".to_string(),
            },
            SideMenu {
                href_path: "./logout".to_string(),
                text: "ログアウト".to_string(),
                icon_path: "".to_string(),
                description: "セッションを終了します。".to_string(),
            },
        ]
    }
}
