// SPDX-License-Identifier: MIT

use anyhow::Result;
use axum::{
    extract::Request,
    http::StatusCode,
    middleware::Next,
    response::{IntoResponse, Response},
    Router,
};
use tower_http::services::ServeDir;
use tower_sessions::{Expiry, SessionManagerLayer};

// common subdirs
mod args;
mod common;
mod error;
mod ip_filter;
mod server;
mod session_store;

// pages
mod connection;
mod container;
mod dhcp;
mod login;
mod nat;
mod nwstatbar;
mod power_management;
mod settings;
mod status;
mod swu;
mod time_settings;
mod top;
mod vpn;
mod wlan;
mod wwan;

#[cfg(debug_assertions)]
const STATIC_RESOURCE_DIR: &str = "static";

#[cfg(not(debug_assertions))]
const STATIC_RESOURCE_DIR: &str = "/usr/share/abos-web/static";

const STATIC_CUSTOMIZE_RESOURCE_DIR: &str = "/etc/atmark/abos_web/customize/static";

async fn fallback_404() -> (StatusCode, &'static str) {
    (StatusCode::NOT_FOUND, "Page not found")
}

// Add text body to some common errors (for curl users who might not
// notice the error)
pub async fn enhance_errors(req: Request, next: Next) -> Response {
    let response = next.run(req).await;

    match response.status() {
        StatusCode::METHOD_NOT_ALLOWED => {
            (StatusCode::METHOD_NOT_ALLOWED, "Method not allowed").into_response()
        }
        _ => response,
    }
}

#[tokio::main]
async fn main() -> Result<()> {
    args::args_init();
    common::password_throttle_init();

    let session_store = session_store::MemoryStore::new(std::time::Duration::from_secs(12 * 3600));
    let session_service = SessionManagerLayer::new(session_store)
        .with_secure(false)
        .with_expiry(Expiry::OnInactivity(time::Duration::hours(12)));

    common::Config::load().await?;

    let app = Router::new()
        .merge(connection::routes())
        .merge(container::routes())
        .merge(dhcp::routes())
        .merge(login::routes())
        .merge(nat::routes())
        .merge(nwstatbar::routes())
        .merge(power_management::routes())
        .merge(settings::routes())
        .merge(status::routes())
        .merge(swu::routes())
        .merge(time_settings::routes())
        .merge(top::routes())
        .merge(vpn::routes())
        .merge(wlan::routes())
        .merge(wwan::routes())
        .nest_service(
            "/static",
            ServeDir::new(STATIC_CUSTOMIZE_RESOURCE_DIR)
                .fallback(ServeDir::new(STATIC_RESOURCE_DIR)),
        )
        .layer(session_service)
        .route_layer(axum::middleware::from_fn(ip_filter::ipaddr_filter))
        .fallback(fallback_404)
        .layer(axum::middleware::from_fn(enhance_errors));

    server::run_server(app).await;
    Ok(())
}
